Package: ptairMS
Title: Pre-processing PTR-TOF-MS Data
Version: 1.21.0
Date: 2020-03-18
Authors@R: person("camille", "Roquencourt", email = "camille.roquencourt@hotmail.fr", role = c("aut", "cre"))
Description: This package implements a suite of methods to preprocess data from PTR-TOF-MS instruments (HDF5 format) and 
   generates the 'sample by features' table of peak intensities in addition to the sample and feature metadata 
   (as a singl<e ExpressionSet object for subsequent statistical analysis). This package also permit usefull tools for cohorts management 
   as analyzing data progressively, visualization tools and quality control. The steps include calibration, expiration detection, 
   peak detection and quantification, feature alignment, missing value imputation and feature annotation. Applications to exhaled air 
   and cell culture in headspace are described in the vignettes and examples. This package was used for data analysis of Gassin Delyle 
   study on adults undergoing invasive mechanical ventilation in the intensive care unit due to severe COVID-19 or non-COVID-19 acute respiratory
   distress syndrome (ARDS), and permit to identfy four potentiel biomarquers of the infection.  
License: GPL-3
Encoding: UTF-8
BugReports: https://github.com/camilleroquencourt/ptairMS/issues
RoxygenNote: 7.2.3
Imports: Biobase, bit64, chron, data.table, doParallel, DT, enviPat,
        foreach, ggplot2, graphics, grDevices, ggpubr, gridExtra,
        Hmisc, methods, minpack.lm, MSnbase, parallel, plotly, rhdf5,
        rlang, Rcpp, shiny, shinyscreenshot, signal, scales, stats,
        utils
Suggests: knitr, rmarkdown, BiocStyle, testthat (>= 2.1.0), ptairData,
        ropls
VignetteBuilder: knitr
LinkingTo: Rcpp
NeedsCompilation: yes
biocViews: Software, MassSpectrometry, Preprocessing, Metabolomics,
        PeakDetection, Alignment
git_url: https://git.bioconductor.org/packages/ptairMS
git_branch: devel
git_last_commit: 258decb
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.24
Date/Publication: 2026-07-14
Packaged: 2026-07-14 23:40:35 UTC; biocbuild
Author: camille Roquencourt [aut, cre]
Maintainer: camille Roquencourt <camille.roquencourt@hotmail.fr>
