# Generated by roxygen2: do not edit by hand

export(LocalMaximaSG)
export(PeakList)
export(RunShinnyApp)
export(alignSamples)
export(annotateVOC)
export(calibration)
export(changeTimeLimits)
export(convert_to_mzML)
export(createPtrSet)
export(defineKnots)
export(detectPeak)
export(exportSampleMetada)
export(formula2mass)
export(getDirectory)
export(getFileNames)
export(getPeakList)
export(getSampleMetadata)
export(importSampleMetadata)
export(impute)
export(imputeMat)
export(plotCalib)
export(plotFeatures)
export(plotRaw)
export(plotTIC)
export(readRaw)
export(resetSampleMetadata)
export(rmPeakList)
export(setSampleMetadata)
export(timeLimits)
export(updatePtrSet)
export(writeEset)
exportClasses(ptrRaw)
exportClasses(ptrSet)
exportMethods(PeakList)
exportMethods(alignSamples)
exportMethods(annotateVOC)
exportMethods(calibration)
exportMethods(defineKnots)
exportMethods(detectPeak)
exportMethods(getFileNames)
exportMethods(plot)
exportMethods(plotCalib)
exportMethods(plotFeatures)
exportMethods(plotRaw)
exportMethods(plotTIC)
exportMethods(show)
exportMethods(timeLimits)
exportMethods(writeEset)
import(Biobase)
import(Rcpp)
import(bit64)
import(data.table)
import(doParallel)
import(foreach)
import(parallel)
importFrom(Rcpp,evalCpp)
importFrom(grDevices,dev.off)
importFrom(grDevices,pdf)
importFrom(graphics,abline)
importFrom(graphics,layout)
importFrom(graphics,par)
importFrom(graphics,title)
importFrom(rlang,.data)
importFrom(scales,hue_pal)
importFrom(shinyscreenshot,screenshotButton)
useDynLib(ptairMS)
