#' Calib_table contains compounds we advice for calibration when exhaled air is 
#' analyzed.
#' 
#' atomic_weights corresponds to the table contained in the url: 
#' http://www.ciaaw.org/atomic-masses.htm
#' 
#' atomic_isotopes correspond to "isotopes" data, contained in the package enviPat,
#' used for isotope matching.
#' 
#' vocDB is the volatolomic data base download on the website : https://hbdb.cmdm.tw, 
#' Human Breathomics Database (search per compounds), used for annotation 
#' and deLacy costallo 14.
#' 
#' ManualAdd add are vocs that we decided to add to vocDB.