# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

findLimDensity <- function(dens, istart, state) {
    .Call('_ptairMS_findLimDensity', PACKAGE = 'ptairMS', dens, istart, state)
}

findEqualGreaterM <- function(x, values) {
    .Call('_ptairMS_findEqualGreaterM', PACKAGE = 'ptairMS', x, values)
}

