% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/SetMethods.R
\name{plot,ptrSet,ANY-method}
\alias{plot,ptrSet,ANY-method}
\alias{plot.ptrSet}
\alias{plot,ptrSet-method}
\title{Plot a ptrSet object}
\usage{
\S4method{plot}{ptrSet,ANY}(x, y, typePlot = "")
}
\arguments{
\item{x}{a ptrSet object}

\item{y}{not use}

\item{typePlot}{could be : \code{calibError}, \code{resolution},
\code{peakShape}, or 
a empty character if you want all.}
}
\value{
plot
}
\description{
plot a ptrSet object
}
\examples{
dirRaw <- system.file("extdata/exhaledAir", package = "ptairData")
exhaledPtrset <- createPtrSet(dir=dirRaw, setName="exhaledPtrset", 
mzCalibRef = c(21.022, 60.0525), fracMaxTIC = 0.7, saveDir = NULL )
plot(exhaledPtrset )
plot(exhaledPtrset ,typePlot='calibError')
plot(exhaledPtrset ,typePlot='resolution')
plot(exhaledPtrset ,typePlot='peakShape')
}
