% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/SetMethods.R
\name{show,ptrSet-method}
\alias{show,ptrSet-method}
\title{show a ptrSet object}
\usage{
\S4method{show}{ptrSet}(object)
}
\arguments{
\item{object}{a ptrSet object}
}
\value{
nothing
}
\description{
It indicates the directory, the numbre of files that contain the directory at 
the moment, and the number of processed files.
The two numbers are different, use \code{updatePtrSet} function.
}
