% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/app.R
\name{changeTimeLimits}
\alias{changeTimeLimits}
\title{Shinny appplication to modify and view expiration limits
 
This function run a shiny app, where you can check the automatic expiration
detection, knots location, and modify it.}
\usage{
changeTimeLimits(ptrSet)
}
\arguments{
\item{ptrSet}{a ptrSet object}
}
\value{
the ptrSet object modified
}
\description{
Shinny appplication to modify and view expiration limits
 
This function run a shiny app, where you can check the automatic expiration
detection, knots location, and modify it.
}
\examples{
library(ptairData)
directory <- system.file("extdata/exhaledAir",  package = "ptairData")
ptrSet <- createPtrSet(directory,setName="ptrSet",mzCalibRef=c(21.022,59.049),
fracMaxTIC=0.8)
\dontrun{ptrSet <- changeTimeLimits(ptrSet)}
}
